<?php

declare(strict_types=1);

namespace Drupal\media_entity_unsplash\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Access\AccessResultNeutral;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\media\Entity\MediaType;
use Drupal\media_entity_unsplash\UnsplashFetcher;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for Media Entity: Unsplash routes.
 */
final class UnsplashAutocompleteController extends ControllerBase {

  /**
   * The controller constructor.
   */
  public function __construct(
    private readonly UnsplashFetcher $unsplashFetcher,
    private readonly Renderer $renderer,
    private readonly AccountProxyInterface $account,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('media_entity_unsplash.unsplash_fetcher'),
      $container->get('renderer'),
      $container->get('current_user'),
    );
  }

  /**
   * Builds the response.
   *
   * @throws \Exception
   */
  public function __invoke(Request $request): JsonResponse {
    $results = [];
    // Get the search string.
    $input = $request->query->get('q');

    // Load the unsplash media type for config values.
    $mediaType = MediaType::load('unsplash');

    // If there is no input, or the media type does not exist, return an
    // empty set.
    if (!$input || !$mediaType) {
      return new JsonResponse($results);
    }

    // Filter the input for security.
    $input = Xss::filter($input);

    // Get the unsplash config.
    $config = $mediaType->get('source_configuration');

    // Set the unsplash credentials.
    $this->unsplashFetcher->setCredentials($config['access_key'], $config['secret_key']);

    // Search for the photos.
    $searchResults = $this->unsplashFetcher->searchPhotos($input);

    // Loop through the photos and create a return array.
    if (!empty($searchResults)) {
      foreach ($searchResults as $result) {
        $label = [];
        // Create the title based on the description, alt or slug.
        $title = $result['description'] ?? $result['alt_description'] ?? $result['slug'];

        // Generate a small thumbnail.
        $image_variables = [
          '#theme' => 'image',
          '#uri' => $result['urls']['small_s3'],
          '#alt' => $result['alt_description'],
          '#title' => $title,
          '#width' => 60,
        ];

        $label[] = $this->renderer->render($image_variables);
        $label[] = $title;

        // Create the output row.
        $results[] = [
          'value' => $result['links']['html'],
          'label' => '<div style="display: flex; flex-wrap: nowrap; align-items: center;">' . implode(' ', $label) . '</div>',
        ];
      }
    }

    // Return the results.
    return new JsonResponse($results);
  }

  /**
   * Check if a user has access on this route.
   *
   * @return \Drupal\Core\Access\AccessResultAllowed|\Drupal\Core\Access\AccessResultNeutral
   *   If a user has access or access is neutral.
   */
  public function checkAccess(): AccessResultNeutral|AccessResultAllowed {
    $allowedPermissions = [
      'create unsplash media',
      'edit any unsplash media',
      'edit own unsplash media',
    ];

    foreach ($allowedPermissions as $permission) {
      if ($this->account->hasPermission($permission)) {
        return AccessResult::allowed();
      }
    }

    return AccessResult::neutral();
  }

}
